@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 module Jekyll
   class Excerpt
     extend Forwardable
@@ -6,8 +8,12 @@ module Jekyll
     attr_accessor :content, :ext
     attr_writer   :output
 
-    def_delegators :@doc, :site, :name, :ext, :relative_path, :extname,
-                          :render_with_liquid?, :collection, :related_posts, :url
+    def_delegators :@doc, :site, :name, :ext, :extname,
+                          :collection, :related_posts,
+                          :coffeescript_file?, :yaml_file?,
+                          :url, :next_doc, :previous_doc
+
+    private :coffeescript_file?, :yaml_file?
 
     # Initialize this Excerpt instance.
     #
@@ -29,8 +35,7 @@ module Jekyll
       @data
     end
 
-    def trigger_hooks(*)
-    end
+    def trigger_hooks(*); end
 
     # 'Path' of the excerpt.
     #
@@ -39,6 +44,13 @@ module Jekyll
       File.join(doc.path, "#excerpt")
     end
 
+    # 'Relative Path' of the excerpt.
+    #
+    # Returns the relative_path for the doc this excerpt belongs to with #excerpt appended
+    def relative_path
+      File.join(doc.relative_path, "#excerpt")
+    end
+
     # Check if excerpt includes a string
     #
     # Returns true if the string passed in
@@ -75,6 +87,10 @@ module Jekyll
       false
     end
 
+    def render_with_liquid?
+      !(coffeescript_file? || yaml_file? || !Utils.has_liquid_construct?(content))
+    end
+
     protected
 
     # Internal: Extract excerpt from the content
@@ -111,14 +127,52 @@ module Jekyll
     # Excerpts are rendered same time as content is rendered.
     #
     # Returns excerpt String
+
+    LIQUID_TAG_REGEX = %r!{%-?\s*(\w+).+\s*-?%}!m
+    MKDWN_LINK_REF_REGEX = %r!^ {0,3}\[[^\]]+\]:.+$!
+
     def extract_excerpt(doc_content)
       head, _, tail = doc_content.to_s.partition(doc.excerpt_separator)
 
+      # append appropriate closing tag (to a Liquid block), to the "head" if the
+      # partitioning resulted in leaving the closing tag somewhere in the "tail"
+      # partition.
+      if head.include?("{%")
+        head =~ LIQUID_TAG_REGEX
+        tag_name = Regexp.last_match(1)
+
+        if liquid_block?(tag_name) && head.match(%r!{%-?\s*end#{tag_name}\s*-?%}!).nil?
+          print_build_warning
+          head << "\n{% end#{tag_name} %}"
+        end
+      end
+
       if tail.empty?
         head
       else
-        "" << head << "\n\n" << tail.scan(/^\[[^\]]+\]:.+$/).join("\n")
+        head.to_s.dup << "\n\n" << tail.scan(MKDWN_LINK_REF_REGEX).join("\n")
       end
     end
+
+    private
+
+    def liquid_block?(tag_name)
+      Liquid::Template.tags[tag_name].superclass == Liquid::Block
+    rescue NoMethodError
+      Jekyll.logger.error "Error:",
+        "A Liquid tag in the excerpt of #{doc.relative_path} couldn't be " \
+        "parsed."
+      raise
+    end
+
+    def print_build_warning
+      Jekyll.logger.warn "Warning:", "Excerpt modified in #{doc.relative_path}!"
+      Jekyll.logger.warn "",
+        "Found a Liquid block containing separator '#{doc.excerpt_separator}' and has " \
+        "been modified with the appropriate closing tag."
+      Jekyll.logger.warn "",
+        "Feel free to define a custom excerpt or excerpt_separator in the document's " \
+        "Front Matter if the generated excerpt is unsatisfactory."
+    end
   end
 end
