@@ -1,5 +1,4 @@
-# encoding: UTF-8
-require 'csv'
+# frozen_string_literal: true
 
 module Jekyll
   class Site
@@ -8,45 +7,58 @@ module Jekyll
                   :exclude, :include, :lsi, :highlighter, :permalink_style,
                   :time, :future, :unpublished, :safe, :plugins, :limit_posts,
                   :show_drafts, :keep_files, :baseurl, :data, :file_read_opts,
-                  :gems, :plugin_manager
+                  :gems, :plugin_manager, :theme
 
     attr_accessor :converters, :generators, :reader
-    attr_reader   :regenerator, :liquid_renderer
+    attr_reader   :regenerator, :liquid_renderer, :includes_load_paths
 
     # Public: Initialize a new Site.
     #
     # config - A Hash containing site configuration details.
     def initialize(config)
-      @config = config.clone
-
-      %w(safe lsi highlighter baseurl exclude include future unpublished
-        show_drafts limit_posts keep_files gems).each do |opt|
-        self.send("#{opt}=", config[opt])
-      end
-
       # Source and destination may not be changed after the site has been created.
-      @source              = File.expand_path(config['source']).freeze
-      @dest                = File.expand_path(config['destination']).freeze
+      @source          = File.expand_path(config["source"]).freeze
+      @dest            = File.expand_path(config["destination"]).freeze
 
-      @reader = Jekyll::Reader.new(self)
-
-      # Initialize incremental regenerator
-      @regenerator = Regenerator.new(self)
+      self.config = config
 
+      @reader          = Reader.new(self)
+      @regenerator     = Regenerator.new(self)
       @liquid_renderer = LiquidRenderer.new(self)
 
-      self.plugin_manager = Jekyll::PluginManager.new(self)
-      self.plugins        = plugin_manager.plugins_path
-
-      self.file_read_opts = {}
-      self.file_read_opts[:encoding] = config['encoding'] if config['encoding']
-
-      self.permalink_style = config['permalink'].to_sym
-
       Jekyll.sites << self
 
       reset
       setup
+
+      Jekyll::Hooks.trigger :site, :after_init, self
+    end
+
+    # Public: Set the site's configuration. This handles side-effects caused by
+    # changing values in the configuration.
+    #
+    # config - a Jekyll::Configuration, containing the new configuration.
+    #
+    # Returns the new configuration.
+    def config=(config)
+      @config = config.clone
+
+      %w(safe lsi highlighter baseurl exclude include future unpublished
+        show_drafts limit_posts keep_files).each do |opt|
+        self.send("#{opt}=", config[opt])
+      end
+
+      # keep using `gems` to avoid breaking change
+      self.gems = config["plugins"]
+
+      configure_plugins
+      configure_theme
+      configure_include_paths
+      configure_file_read_opts
+
+      self.permalink_style = config["permalink"].to_sym
+
+      @config
     end
 
     # Public: Read, process, and write this Site to output.
@@ -59,27 +71,32 @@ module Jekyll
       render
       cleanup
       write
-      print_stats
+      print_stats if config["profile"]
     end
 
     def print_stats
-      if @config['profile']
-        puts @liquid_renderer.stats_table
-      end
+      Jekyll.logger.info @liquid_renderer.stats_table
     end
 
     # Reset Site details.
     #
     # Returns nothing
     def reset
-      self.time = (config['time'] ? Utils.parse_date(config['time'].to_s, "Invalid time in _config.yml.") : Time.now)
+      self.time = if config["time"]
+                    Utils.parse_date(config["time"].to_s, "Invalid time in _config.yml.")
+                  else
+                    Time.now
+                  end
       self.layouts = {}
       self.pages = []
       self.static_files = []
       self.data = {}
+      @site_data = nil
       @collections = nil
+      @docs_to_write = nil
       @regenerator.clear_cache
       @liquid_renderer.reset
+      @site_cleaner = nil
 
       if limit_posts < 0
         raise ArgumentError, "limit_posts must be a non-negative number"
@@ -106,18 +123,21 @@ module Jekyll
       dest_pathname = Pathname.new(dest)
       Pathname.new(source).ascend do |path|
         if path == dest_pathname
-          raise Errors::FatalException.new "Destination directory cannot be or contain the Source directory."
+          raise Errors::FatalException,
+            "Destination directory cannot be or contain the Source directory."
         end
       end
     end
 
     # The list of collections and their corresponding Jekyll::Collection instances.
-    # If config['collections'] is set, a new instance is created for each item in the collection.
-    # If config['collections'] is not set, a new hash is returned.
+    # If config['collections'] is set, a new instance is created
+    # for each item in the collection, a new hash is returned otherwise.
     #
     # Returns a Hash containing collection name-to-instance pairs.
     def collections
-      @collections ||= Hash[collection_names.map { |coll| [coll, Jekyll::Collection.new(self, coll)] } ]
+      @collections ||= Hash[collection_names.map do |coll|
+        [coll, Jekyll::Collection.new(self, coll)]
+      end]
     end
 
     # The list of collection names.
@@ -125,11 +145,11 @@ module Jekyll
     # Returns an array of collection names from the configuration,
     #   or an empty array if the `collections` key is not set.
     def collection_names
-      case config['collections']
+      case config["collections"]
       when Hash
-        config['collections'].keys
+        config["collections"].keys
       when Array
-        config['collections']
+        config["collections"]
       when nil
         []
       else
@@ -151,7 +171,10 @@ module Jekyll
     # Returns nothing.
     def generate
       generators.each do |generator|
+        start = Time.now
         generator.generate(self)
+        Jekyll.logger.debug "Generating:",
+          "#{generator.class} finished in #{Time.now - start} seconds."
       end
     end
 
@@ -165,25 +188,10 @@ module Jekyll
 
       Jekyll::Hooks.trigger :site, :pre_render, self, payload
 
-      collections.each do |_, collection|
-        collection.docs.each do |document|
-          if regenerator.regenerate?(document)
-            document.output = Jekyll::Renderer.new(self, document, payload).run
-            document.trigger_hooks(:post_render)
-          end
-        end
-      end
-
-      pages.flatten.each do |page|
-        if regenerator.regenerate?(page)
-          page.output = Jekyll::Renderer.new(self, page, payload).run
-          page.trigger_hooks(:post_render)
-        end
-      end
+      render_docs(payload)
+      render_pages(payload)
 
       Jekyll::Hooks.trigger :site, :post_render, self, payload
-    rescue Errno::ENOENT
-      # ignore missing layout dir
     end
 
     # Remove orphaned files and empty directories in destination.
@@ -205,7 +213,7 @@ module Jekyll
     end
 
     def posts
-      collections['posts'] ||= Collection.new(self, 'posts')
+      collections["posts"] ||= Collection.new(self, "posts")
     end
 
     # Construct a Hash of Posts indexed by the specified Post attribute.
@@ -225,17 +233,19 @@ module Jekyll
       # Build a hash map based on the specified post attribute ( post attr =>
       # array of posts ) then sort each array in reverse order.
       hash = Hash.new { |h, key| h[key] = [] }
-      posts.docs.each { |p| p.data[post_attr].each { |t| hash[t] << p } if p.data[post_attr] }
-      hash.values.each { |posts| posts.sort!.reverse! }
+      posts.docs.each do |p|
+        p.data[post_attr].each { |t| hash[t] << p } if p.data[post_attr]
+      end
+      hash.each_value { |posts| posts.sort!.reverse! }
       hash
     end
 
     def tags
-      post_attr_hash('tags')
+      post_attr_hash("tags")
     end
 
     def categories
-      post_attr_hash('categories')
+      post_attr_hash("categories")
     end
 
     # Prepare site data for site payload. The method maintains backward compatibility
@@ -243,7 +253,7 @@ module Jekyll
     #
     # Returns the Hash to be hooked to site.data.
     def site_data
-      config['data'] || data
+      @site_data ||= (config["data"] || data)
     end
 
     # The Hash payload containing site-wide data.
@@ -267,10 +277,12 @@ module Jekyll
     # Get the implementation class for the given Converter.
     # Returns the Converter instance implementing the given Converter.
     # klass - The Class of the Converter to fetch.
-
     def find_converter_instance(klass)
-      converters.find { |klass_| klass_.instance_of?(klass) } || \
-        raise("No Converters found for #{klass}")
+      @find_converter_instance ||= {}
+      @find_converter_instance[klass] ||= begin
+        converters.find { |converter| converter.instance_of?(klass) } || \
+          raise("No Converters found for #{klass}")
+      end
     end
 
     # klass - class or module containing the subclasses.
@@ -289,11 +301,11 @@ module Jekyll
     #
     # Returns
     def relative_permalinks_are_deprecated
-      if config['relative_permalinks']
+      if config["relative_permalinks"]
         Jekyll.logger.abort_with "Since v3.0, permalinks for pages" \
                                 " in subfolders must be relative to the" \
                                 " site source directory, not the parent" \
-                                " directory. Check http://jekyllrb.com/docs/upgrading/"\
+                                " directory. Check https://jekyllrb.com/docs/upgrading/"\
                                 " for more info."
       end
     end
@@ -302,7 +314,7 @@ module Jekyll
     #
     # Returns an Array of Documents which should be written
     def docs_to_write
-      documents.select(&:write?)
+      @docs_to_write ||= documents.select(&:write?)
     end
 
     # Get all the documents
@@ -334,7 +346,7 @@ module Jekyll
     #
     # Returns a Boolean: true for a full rebuild, false for normal build
     def incremental?(override = {})
-      override['incremental'] || config['incremental']
+      override["incremental"] || config["incremental"]
     end
 
     # Returns the publisher or creates a new publisher if it doesn't
@@ -357,6 +369,19 @@ module Jekyll
       end
     end
 
+    # Public: Prefix a given path with the theme directory.
+    #
+    # paths - (optional) path elements to a file or directory within the
+    #         theme directory
+    #
+    # Returns a path which is prefixed with the theme root directory.
+    def in_theme_dir(*paths)
+      return nil unless theme
+      paths.reduce(theme.root) do |base, path|
+        Jekyll.sanitized_path(base, path)
+      end
+    end
+
     # Public: Prefix a given path with the destination directory.
     #
     # paths - (optional) path elements to a file or directory within the
@@ -369,11 +394,19 @@ module Jekyll
       end
     end
 
-    private
+    # Public: The full path to the directory that houses all the collections registered
+    # with the current site.
+    #
+    # Returns the source directory or the absolute path to the custom collections_dir
+    def collections_path
+      dir_str = config["collections_dir"]
+      @collections_path ||= dir_str.empty? ? source : in_source_dir(dir_str)
+    end
 
     # Limits the current posts; removes the posts which exceed the limit_posts
     #
     # Returns nothing
+    private
     def limit_posts!
       if limit_posts > 0
         limit = posts.docs.length < limit_posts ? posts.docs.length : limit_posts
@@ -385,8 +418,66 @@ module Jekyll
     # already exist.
     #
     # Returns The Cleaner
+    private
     def site_cleaner
       @site_cleaner ||= Cleaner.new(self)
     end
+
+    private
+    def configure_plugins
+      self.plugin_manager = Jekyll::PluginManager.new(self)
+      self.plugins        = plugin_manager.plugins_path
+    end
+
+    private
+    def configure_theme
+      self.theme = nil
+      return if config["theme"].nil?
+
+      self.theme =
+        if config["theme"].is_a?(String)
+          Jekyll::Theme.new(config["theme"])
+        else
+          Jekyll.logger.warn "Theme:", "value of 'theme' in config should be " \
+          "String to use gem-based themes, but got #{config["theme"].class}"
+          nil
+        end
+    end
+
+    private
+    def configure_include_paths
+      @includes_load_paths = Array(in_source_dir(config["includes_dir"].to_s))
+      @includes_load_paths << theme.includes_path if theme && theme.includes_path
+    end
+
+    private
+    def configure_file_read_opts
+      self.file_read_opts = {}
+      self.file_read_opts[:encoding] = config["encoding"] if config["encoding"]
+      self.file_read_opts = Jekyll::Utils.merged_file_read_opts(self, {})
+    end
+
+    private
+    def render_docs(payload)
+      collections.each_value do |collection|
+        collection.docs.each do |document|
+          render_regenerated(document, payload)
+        end
+      end
+    end
+
+    private
+    def render_pages(payload)
+      pages.flatten.each do |page|
+        render_regenerated(page, payload)
+      end
+    end
+
+    private
+    def render_regenerated(document, payload)
+      return unless regenerator.regenerate?(document)
+      document.output = Jekyll::Renderer.new(self, document, payload).run
+      document.trigger_hooks(:post_render)
+    end
   end
 end
