@@ -1,9 +1,12 @@
-require 'helper'
+# frozen_string_literal: true
+
+require "helper"
 
 class TestLayoutReader < JekyllUnitTest
   context "reading layouts" do
     setup do
-      config = Jekyll::Configuration::DEFAULTS.merge({'source' => source_dir, 'destination' => dest_dir})
+      config = Jekyll::Configuration::DEFAULTS.merge({ "source"      => source_dir,
+                                                       "destination" => dest_dir, })
       @site = fixture_site(config)
     end
 
